void draw()
{
  // for animation
  float fTimeDiff = (float)(millis() - gnTimeStart)/1000.0f;

  // select data by current mode
  int [][]pData = null;
  int nDataMax = 0;
  String strTitle = "";
  float RGB_R1 = 0.0f;
  float RGB_G1 = 0.0f;
  float RGB_B1 = 0.0f;
  float RGB_R2 = 0.0f;
  float RGB_G2 = 0.0f;
  float RGB_B2 = 0.0f;

  switch(heightMode) {

  case 1:
    pData = gNYTimesFreq;
    nDataMax = gNYTimesFreqMax;
    strTitle = "NYTimes: South Korea | North Korea ("+gnYearStart+"-"+gnYearEnd+")";
    //south(blue)
    RGB_R1 = 0.0f;
    RGB_G1 = 0.1f;
    RGB_B1 = 1.0f;
    //north(red)
    RGB_R2 = 1.0f;
    RGB_G2 = 0.0f;
    RGB_B2 = 0.0f;
    break;

  case 2:
    pData = gSPLFreq;
    nDataMax = gSPLFreqMax;
    strTitle = "SPL: South Korea | North Korea ("+gnYearStart+"-"+gnYearEnd+")";
     //south(blue)
    RGB_R1 = 0.0f;
    RGB_G1 = 0.4f;
    RGB_B1 = 1.0f;
    //north(red)
    RGB_R2 = 1.0f;
    RGB_G2 = 0.2f;
    RGB_B2 = 0.5f;
    break;

  default:
    assert(false);
    break;
  }


  // draw start
  
  pushMatrix();
  pushStyle();

  colorMode(RGB, 1.0f);
  background(0.0f);
  fill(.9f);
  textFont(myFont);
  textAlign(LEFT, CENTER);
  text(strTitle, 50, 40);

  // # of bars and bar sizes
  int   nItems = (gnYearEnd-gnYearStart+1)*12;
  float fWidth = (float)(width-gfHMargin*2)/(float)nItems;
  float fHeight = (float)(height-gfVMargin*2)/2.0f;
  float fAnimationRatio = fTimeDiff / gfAnimationSpeed;
  if ( fAnimationRatio > 1.0f ) fAnimationRatio = 1.0f;
  
  // background gradient images
  float fBgBoxWidth = (width-gfHMargin*2)/float(gnYearEnd-gnYearStart+1);
  float fBgBoxHeight = (height-gfVMargin*2)/2.0f;

  noStroke();  
  for (int y=0; y<gnYearEnd-gnYearStart+1; y++ )  {
    float fBgBoxOffsetX = gfHMargin + fBgBoxWidth * y;
    float fBgBoxOffsetY = gfVMargin;
    float fBgBoxSliceWidth = fBgBoxWidth / float(gnBgBoxGradientLevel);
    
    for (int x=0; x<gnBgBoxGradientLevel; x++ )  {
      
      fill( 1.0f, 1.0f, 1.0f, .09f * ( 1.0f - 1.0f/float(gnBgBoxGradientLevel)*float(x) ) * fAnimationRatio );
      rect( fBgBoxOffsetX + fBgBoxSliceWidth*x, height/2, fBgBoxSliceWidth, +fBgBoxHeight );
      fill( 1.0f, 1.0f, 1.0f, .09f * ( 1.0f - 1.0f/float(gnBgBoxGradientLevel)*float(x) ) * fAnimationRatio );
      rect( fBgBoxOffsetX + fBgBoxSliceWidth*x, height/2, fBgBoxSliceWidth, -fBgBoxHeight );
      
    }
    
    textFont(myFont3);
    textAlign(LEFT, CENTER);
    fill(1.0f, 1.0f, 1.0f, .2f);
    text(str(gnYearStart+y), fBgBoxOffsetX, height/2 + fBgBoxHeight * .95f);
    //fill(1.0f, 1.0f, 1.0f, .5f);
    //text(str(gnYearStart+y), fBgBoxOffsetX, height/2 - fBgBoxHeight * .95f);
    
  }

  // draw data
  for (int n = 0; n < nItems; n++ ) {

    float fX = gfHMargin + fWidth * n;
    float fY = height/2;
    float fHeightN = -pData[0][n]/float(nDataMax)*fHeight * fAnimationRatio;
    float fHeightS = +pData[1][n]/float(nDataMax)*fHeight * fAnimationRatio;

    // south korea
    noStroke();
    fill(RGB_R1, RGB_G1, RGB_B1, .3f + .7f * (float)pData[1][n]/(float)nDataMax * fAnimationRatio);
    rect( fX, fY, fWidth, fHeightS );

    // north korea
    noStroke();
    fill(RGB_R2, RGB_G2, RGB_B2, .3f + .7f * (float)pData[0][n]/(float)nDataMax * fAnimationRatio);
    rect( fX, fY, fWidth, fHeightN );
  }

  // mouse event (on mouse over)
  if ( gbMouseEvent == true ) {
    
    float fX = gfHMargin + fWidth * gnMouseEventBarX;
    float fY = height/2;
    float fHeightEvent = .0f;
    float fHeightEventText = .0f;
    String strText = str(pData[gnMouseEventBarY][gnMouseEventBarX])+" article(s)\n"+str(2005+gnMouseEventBarX/12)+"."+str(gnMouseEventBarX%12+1);

    fHeightEvent = pData[gnMouseEventBarY][gnMouseEventBarX]/float(nDataMax)*fHeight * fAnimationRatio;
    fHeightEventText = fHeightEvent + 20;
    if ( gnMouseEventBarY == 0 ) {
      fHeightEvent = -fHeightEvent;
      fHeightEventText = -fHeightEventText;
    }
    if ( gnMouseEventBarY == 1 ) {
      fHeightEvent = fHeightEvent;
      fHeightEventText = fHeightEventText;
    }

    stroke(1.0f);
    if ( gnMouseEventBarY == 1 )  fill(.0f, .0f, 1.0f, 1.0f);
    if ( gnMouseEventBarY == 0 )  fill(1.0f, .0f, .0f, 1.0f);

    rect( fX, fY, fWidth, fHeightEvent );

    textFont(myFont2);
    textAlign(CENTER, CENTER);
    textLeading(10);
    fill( 1.0f);
    text(strText, fX + fWidth/2, fY+fHeightEventText);
    
  }

  popMatrix();
  popStyle();
}

